import os
from os.path import join, dirname, splitext, isabs
import time
from typing import Callable, Awaitable, Dict, Any, List, Set, Union, Tuple
from logging import getLogger, Logger
import json
import asyncio
import logging
from os.path import exists

from aiohttp import web

from agi_green.dispatcher import Protocol, format_call, protocol_handler

here = dirname(__file__)
logger = logging.getLogger(__name__)
log_level = os.getenv('LOG_LEVEL', 'WARNING').upper()
logging.basicConfig(level=log_level)


WS_PING_INTERVAL = 20

class WebSocketProtocol(Protocol):
    '''
    Websocket session
    '''
    protocol_id: str = 'ws'

    def __init__(self, parent:Protocol):
        super().__init__(parent)
        self.sockets: Set[web.WebSocketResponse] = set()
        self.pre_connect_queue = []

    async def ping_loop(self, socket: web.WebSocketResponse):
        'ping the websocket to keep it alive'
        last_pong_time = time.time()

        while socket in self.sockets:
            try:
                await socket.ping()
            except ConnectionResetError as e:
                logger.error(f'ws connection reset (closing) {e} {self.dispatcher.session_id}')
                self.sockets.discard(socket)
                break
            await asyncio.sleep(WS_PING_INTERVAL)

    async def do_send(self, cmd: str, **kwargs):
        'send ws message to all connected browsers via websocket'
        kwargs['cmd'] = cmd
        if self.sockets:
            try:
                s = json.dumps(kwargs)
            except Exception as e:
                logger.error(f'ws send error: {e})')
                logger.error(f'ws send error: {kwargs}')
                return

            dead_sockets = set()
            for socket in self.sockets:
                try:
                    await socket.send_str(s)
                except Exception as e:
                    logger.error(f'ws send error: {e} (removing socket)')
                    dead_sockets.add(socket)

            self.sockets -= dead_sockets
            if not self.sockets:
                self.pre_connect_queue.append(kwargs)
        else:
            logger.info(f'queuing ws: {format_call(cmd, kwargs)}')
            self.pre_connect_queue.append(kwargs)

    @protocol_handler
    async def on_ws_connect(self, socket: web.WebSocketResponse):
        'websocket connected'
        if not self.is_server:
            self.sockets.add(socket)

            while self.pre_connect_queue and self.sockets:
                kwargs = self.pre_connect_queue.pop(0)
                await self.do_send(**kwargs)

            if not self.sockets:
                logger.error('all websockets closed before queue was emptied')
                return

            self.add_task(self.ping_loop(socket))

    @protocol_handler
    async def on_ws_disconnect(self, socket: web.WebSocketResponse):
        'websocket disconnected'
        self.sockets.discard(socket)




import logging
from typing import Dict, Any
import aiohttp
from agi_green.dispatcher import Protocol, protocol_handler

logger = logging.getLogger(__name__)

class AzureProtocol(Protocol):
    """
    Azure protocol handler for managing Azure-specific user identification
    """
    protocol_id: str = 'azure'

    def __init__(self, parent: Protocol):
        super().__init__(parent)
        self.azure_user = "Unknown User"
        self.azure_id = "Unknown ID"

    async def get_user_photo(self, access_token: str) -> str:
        """
        Fetch user's photo from Microsoft Graph API
        Returns photo URL or default avatar if unavailable
        """
        try:
            async with aiohttp.ClientSession() as session:
                headers = {'Authorization': f'Bearer {access_token}'}
                url = f'https://graph.microsoft.com/v1.0/me/photo/$value'
                async with session.get(url, headers=headers) as response:
                    if response.status == 200:
                        # Here you'd need to actually save the photo somewhere and return its URL
                        # For now, we'll just return the default
                        return '/avatars/azure.png'
                    else:
                        logger.error(f"Failed to fetch azure user photo: {response.status}")
        except Exception as e:
            logger.error(f"Failed to fetch user photo: {e}")
        
        return '/avatars/azure.png'

    @protocol_handler
    async def on_ws_connect(self, headers: Dict[str, str]):
        """
        Handle WebSocket connection and extract Azure user information from headers
        """
        self.azure_user = headers.get("X-MS-CLIENT-PRINCIPAL-NAME", None)
        self.azure_id = headers.get("X-MS-CLIENT-PRINCIPAL-ID", None)
        access_token = headers.get("X-MS-TOKEN-AAD-ACCESS-TOKEN", None)

        if self.azure_user and self.azure_id:
            logger.info(f"Azure User Connected - Name: {self.azure_user}, ID: {self.azure_id}")
            
            if access_token:
                icon = await self.get_user_photo(access_token)
            else:
                logger.warning("No access token found for Azure user")
                icon = '/avatars/azure.svg'

            await self.send('ws', 'set_user_data', 
                            uid=self.azure_id, 
                            name=self.azure_user, 
                            icon=icon)

